#include "Ti86asm.inc"  ; TI System Information

; TriPeaks for TI-86
; by Andy Kornaus
; Based on TriPeaks for Windows Entertainment Pack
; Refer to text file for how to play
; You may look at this code but DON'T COPY IT unless you have my permission
; I commented some things for your convienience.
; Any questions e-mail racraccoon@yahoo.com

; The card layout:
; .db %00ssffff
;   ss = suit, ffff = face
;    Suits          Faces
;   00 Clubs    |  0000 None   0100 4      1000 8      1100 Queen
;   01 Diamonds |  0001 Ace    0101 5      1001 9      1101 King
;   10 Spades   |  0010 2      0110 6      1010 10     1110 Ace high*
;   11 Hearts   |  0011 3      0111 7      1011 Jack   1111 Joker*
; * Not used.

; Bitmasks for the cards ~ *AND* these with a card to get your Value...
Face        equ     %00001111
Suit        equ     %00110000
; Values for the cards ~ *CP* these with a bitmasked card to test.
Clubs       equ     %00000000
Diamonds    equ     %00010000
Spades      equ     %00100000
Hearts      equ     %00110000
None        equ     %00000000
Ace         equ     %00000001
Two         equ     %00000010   ; The number ones are kind of redundant
Three       equ     %00000011
Four        equ     %00000100
Five        equ     %00000101
Six         equ     %00000110
Seven       equ     %00000111
Eight       equ     %00001000
Nine        equ     %00001001
Ten         equ     %00001010
Jack        equ     %00001011
Queen       equ     %00001100
King        equ     %00001101
;AceHigh     equ    %00001110   ; These aren't used in TriPeaks
;Joker       equ    %00001111

.org _asm_exec_ram    ; $D748

Main:
    call    _flushallmenus      ; Close all open menus
    call    _runindicoff        ; Turn off the annoying run indicator
    call    _clrLCD             ; Clear the screen
    set     3,      (IY+apdflags)   ; Enables APD to run durning the program
    ld      de,     $0A0A
    ld      bc,     $246C
    call    DrawRect
    ld      hl,         $0503
    ld      (_curRow),  hl
    ld      hl,         Title
    call    _puts
Pause:
    call    _get_key
    cp      K_EXIT
    jp      z,      Quit
    cp      K_ENTER
    jr      nz,     Pause

    ld      hl,     $0000
    ld      (DumpR),    hl
    ld      (CurPosn),hl
    ld      hl,     SaveName
    rst     20h                 ; Put "tpsave" in OP1
    rst     10h                 ; Search for "tpsave"
    jr      c,      Reset       ; If it doesn't exist, start a new game.
    ld      a,      b           ; BDE = abs addr, move it to AHL
    ex      de,     hl
    call    _ahl_plus_2_pg3     ; AHL + 2
    call    _set_abs_src_addr   ; This is our source
    xor     a                   ; Clear A so it doesn't interfere in AHL.
    ld      hl,     SaveGame    ; AHL = SaveGame
    call    _set_abs_dest_addr  ; This is our destination
    ld      hl,     tempData - SaveGame ; Length of the saved game
    call    _set_mm_num_bytes   ; Sets length
    call    _mm_ldir            ; Safe LDIR: will copy even if the file
                                ; spills onto another page.
    jp      Display             ; Game is loaded; display it

Reset:
    ld      hl,     $0000
    ld      (Money),hl
    jr      GenCards            ; May as well jump past the penalty, since it's reset.
NewGame:
    ld      a,      (DeckLen)   ; See how many cards are still in the deck
    or      a
    jr      z,      GenCards    ; Nothing left? OK to start new game
    xor     a                   ; A will hold the $$ of the penalty
    ld      hl,     Posn1       ; Check our positions
    ld      b,      28          ; Must set up a loop for how many cards left on table
Penalty:
    ld      c,      (hl)
    inc     hl
    inc     c   \   dec     c   ; Affect the flags
    jr      z,      NoPenalty
    add     a,      5
NoPenalty:
    djnz    Penalty
    ld      c,      a
  ;  ld      b,      0
    ld      hl,     (Money)
    sbc     hl,     bc
    ld      (Money),hl
GenCards:
    ; Generate the cards...place them in our Scrap area
    ld      hl,     ScrapL
    ld      (hl),   52
    inc     hl
    ld      a,      $30       ; A holds the suit, $30, $20, $10, $00
MakeCards:
    ld      b,      13        ; B holds the face, 13-1
    ld      c,      a         ; C is a backup of A
MakeLoop:
    or      b                 ; A is now a card
    ld      (hl),   a         ; Put it in the scrap deck
    inc     hl
    ld      a,      c         ; Restore current suit
    djnz    MakeLoop
    sub     $10               ; Next suit
    jr      nc,     MakeCards ; If A carries, all 4 suits are done
        
Deal:
    ld      hl,     ScrapL
    ld      a,      (hl)
    ld      b,      a
    push    af
    call    Random              ; B = 0 after call to Random
    ld      d,      b           ; So, this assigns 0 to D
    ld      e,      a           ; DE = random #
    pop     af                  ; B holds the deck length
    dec     a                   ; Length - 1
    ld      (hl),   a           ; Store updated length
    push    af                  ; Store for later
    sub     e                   ; E holds random #
    ld      c,      a           ; BC = length - random #, or...
 ;  ld      b,      0           ; BC = how long we need to shift
    inc     hl                  ; HL = Scrap Deck (it was one byte previous)
    add     hl,     de          ; Add the random offset (1 through x)
    ld      a,      (hl)        ; A = card at this point in the deck
    ld      d,      h           ; DE = this point
    ld      e,      l
    inc     hl                  ; HL = one byte ahead
    inc     c   \   dec	    c   ; This affects the flags so we can tell if C is zero
    jr      z,      $+4         ; If so, jump over LDIR (otherwise, it uncontrollably copies data)
    ldir                        ; Shifts the remaining cards over
    ld      c,      a           ; This is now the current card
    pop     af
    ld      e,      a           ; DE = length of Scrap Deck
    ld      d,      b           ; Yeah, B = 0 again
    ld      hl,     Posn1       ; Point to the first position
    add     hl,     de          ; When we add these, we'll get an offset that
    ld      (hl),   c           ; eventually equals 0. (It deals backwards)
    or      a
    jr      nz,     Deal

    dec     hl                  ; HL = deck length
    ld      a,      23
    ld      (hl),   a
    dec     hl                  ; HL = Current $$ value
    inc     b                   ; B is still 0 from earlier, so INC it to get 1.
    ld      (hl),   b

Display:
    call    _clrLCD
    ld      hl,     PosnData-1
    ld      bc,     Posn1
    push    af
NextRow:
    inc     hl
    pop     de
    ld      d,      (hl)        ; This retrieves the y value for the row
    push    de
    inc     hl
DispLoop:
    pop     de
    push    de
    inc     hl
    ld      a,      (bc)
    or      a                   ; If the value is zero, there's no card.
    jp      z,      NextCard    ; Skip to the next card if so.
    dec     hl
    push    bc                  ; There is definitely a card here, so
    push    hl                  ; set up coords and draw rectangle round it.
    ld      e,      (hl)
    ld      bc,     $0F0A       ; $0F = 15 tall, $0A = 10 wide
    push    de
    call    DrawRectOutside
    pop     de                  ; Now to see if this card is covered.
    pop     hl
    pop     bc
    inc     hl
    push    bc
    push    hl
    ld      l,      (hl)        ; covers this one up.
    ld      h,      0           ; Here is the offset to the card that
    add     hl,     bc          ; HL = 1st card that covers it.
    ld      a,      (hl)        ; The value.
    inc     hl                  ; HL = 2nd card that covers it.
    ld      b,      (hl)        ; Its value as well.
    cp      b                   ; Cards can't be equal unless they're both zero
    jr      z,      DispCard    ; If the card isn't covered, display it.
    ld      b,      13
    ld      hl,     CardBack
    call    PutSprite
    jr      popNext
DispCard:
    pop     hl
    pop     bc
    ld      a,      (bc)
    push    bc
    push    hl
    push    de
    push    af
;   and     Suit        ; I leave this out because after _SHRACC shifts,
    call    _shracc     ; it does AND $0F, and nothing should be there that shouldn't
    ld      b,      a
    rla                     ; A * 2
    add     a,      b       ; A * 3
    rla                     ; A * 6
    ld      hl,     SuitSprites
    ld      b,      a
    jr      z,      aClub   ; The last RLA would give the zero flag.
suitPtr:
    inc     hl
    djnz    suitPtr
aClub:
    pop     bc
    pop     de
    push    bc
    ld      a,      d
    add     a,      7
    ld      d,      a
    push    de
    ld      b,      6
    call    PutSprite
    pop     de
    pop     bc
    ld      a,      d
    sub     7
    ld      d,      a
    ld      a,      e
    inc     a
    ld      e,      a
    ld      (_penCol),de
    ld      a,      b
    ld      hl,     Faces
    and     Face
    dec     a           ; This will tell us right away if it's an Ace
    jr      z,      dFace
    sub     9           ; If C, it's a number; if Z, it's 10
    jr      c,      aNum
    jr      z,      put10
    ld      b,      a   ; At this point, it's J, Q, or K
facePtr:
    inc     hl
    djnz    facePtr
dFace:
    ld      a,      (hl)    ; HL points to A, J, Q, or K
    jr      putFace
put10:
    ld      a,      $81 ; Sub 1, looks nicer in vfont than '1'.
    call    _vputmap
    ld      a,      $F6 ; When we add $3A, it will be $30 or '0'
aNum:
    add     a,      $3A ; Add 10 back in ($A) and $30 to get ASCII value
putFace:
    call    _vputmap
popNext:
    pop     hl
    pop     bc
NextCard:
    inc     hl
    inc     bc
    ld      a,      (hl)
    or      a
 ;   jp      z,      EndDLoop
 ;   cp      $D6
    jp      m,      NextRow ; If negative (bit 7 set), it's too large to be a co-ord.
    jp      nz,     DispLoop ;If zero, fall out to continue
    ; Now we finished displaying the TriPeaks and the pile.
    ; On to the deck and indicators (Deck, Money).
EndDLoop:
    pop     af
    ld      a,      (DeckLen)
    or      a
    jr      z,      SkipDeck
    push    af                  ; Save the length of the deck
    ld      de,     $2906       ; Coordinates
    ld      bc,     $0F0A       ; Length / Width
    push    de
    call    DrawRectOutside
    pop     de
    ld      b,      13
    ld      hl,     CardBack
    call    PutSprite
    pop     af                  ; Recall the length of the deck
SkipDeck:
    ld      de,     $3805
    ld      (_penCol),de
 ;  ld      a,      (DeckLen)   ; A = (DeckLen) from earlier
    call    Disp_A
    ld      de,     $3862
    ld      (_penCol),de
    ld      a,      $24         ; Code for dollar sign
    call    _vputmap
    ld      hl,     (Money)
    call    DispHL
    
    call    ClearedTriPeaks     ; Check if the game is over
    jr      nz,     DispCursor  ; If so, display new game text
    ld      de,     $1927
    ld      (_penCol),de
    ld      hl,     GameOver
    call    _vputs
DispCursor:
    ld      de,     (CurPosn)
 ;   ld      hl,     Cursor
 ;   ld      b,      10
 ;   call    PutSprite
    call    DrawCursor

KeyInput:
    ld      a,      $FF         ; After any key is pressed, reset the keypress.
    ld      (KeyPres),a         ; Reset the stored keypress.
    call    _resetAPD           ; Reset the APD timer
Delay:
 ;   ei
    halt
    halt
    halt
    halt
    halt
  ;  di

KeyLoop:
    ld      a,      %11111111   ; Reset Key Port
    out     (1),    a
    ld      a,      %10111111   ; Check for MORE, EXIT, 2ND, F1, F2, F3, F4, F5
    ld      b,      a           ; Save our key group
    out     (1),    a           ; Send bitmask
    in      a,      (1)         ; Read key value
    cp      %11111111           ; Check if anything was pressed
    jr      nz,     StoKeyPress ; Something was pressed? Store it
    ld      a,      %11011111   ; Check for ALPHA, GRAPH, LOG, LN, x^2, ,, STO>
    ld      b,      a
    out     (1),    a
    in      a,      (1)
    cp      %11111111
    jr      nz,     StoKeyPress
    ld      a,      (KeyGrp)    ; At this point, nothing was pressed. Check last key pressed.
    cp      %10111111           ; 2nd Group
    jr      nz,     NextGroup   ; No? Check another
    ld      a,      (KeyPres)   ; Last stored keypress.
    cp      %10111111           ; EXIT was pressed
    jp      z,      Exit
    cp      %11011111           ; 2nd was pressed
    jr      z,      CursorInput
    cp      %11101111           ; F1 was pressed
    jp      z,      NewGame
    cp      %11110111           ; F2 was pressed
    jp      z,      Reset
    cp      %11111110           ; F5 was pressed
    jp      z,      Quit
NextGroup:
    ld      a,      (KeyGrp)
    cp      %11011111           ; Alpha Group
    jr      nz,     Arrows      ; No? No key of interest was pressed. Proceed to arrows.
    ld      a,      (KeyPres)   ; Get the last stored keypress.
    cp      %01111111           ; ALPHA was pressed
    jr      z,      DeckIn
 ;   inc     a                   ; This will carry when it equals $FF
 ;   jr      nc,     KeyInput    ; If an unused key was being pressed, reset the APD counter.
Arrows:
    ld      a,      %11111110   ; Check for UP, RIGHT, LEFT, DOWN
    out     (1),    a
    in      a,      (1)
    ld      c,      a
    bit     0,      a           ; Down was pressed
    jr      z,      Down
    bit     1,      a           ; Left was pressed
    jr      z,      Left
    bit     2,      a           ; Right was pressed
    jr      z,      Right
    bit     3,      a           ; Up was pressed
    jr      z,      Up
    jr      Delay           ; Go back to Delay instead of KeyLoop
    ; The delay is unnoticable to the user and saves the batteries a little :)

StoKeyPress:
    ld      (KeyPres),a
    ld      a,      b
    ld      (KeyGrp),a
    jr      Arrows

Down:
    call    MoveDown
    bit     1,      c
    call    z,      MoveLeft
    bit     2,      c
    call    z,      MoveRight
    jp      KeyInput

Left:
    call    MoveLeft
    bit     3,      c
    call    z,      MoveUp
    jp      KeyInput

Right:
    call    MoveRight
    bit     3,      c
    call    z,      MoveUp
    jp      KeyInput

Up:
    call    MoveUp
    jp      KeyInput

CursorInput:
    ; Before we check anything, check if the user clicked the deck.
    ld      bc,     (CurPosn)
    ld      a,      $29
    cp      b
    jr      nc,     CursorIn
    ld      a,      $35
    cp      b
    jr      c,      CursorIn
    ld      a,      $5
    cp      c
    jr      nc,     CursorIn
    ld      a,      $C
    cp      c
    jr      c,      CursorIn
DeckIn:
    ld      a,      (DeckLen)
    ld      c,      a
    or      a                       ; If length = 0, no cards to draw
    jp      z,      KeyInput
    call    ClearedTriPeaks         ; If game's over, no need to draw
    jp      z,      KeyInput
    ld      de,     Pile
    ld      hl,     Deck
    ld      b,      0
    ld      a,      c
    ldir
    dec     a
    ld      (DeckLen),a
    ld      hl,     (Money)
    ld      de,     5
    sbc     hl,     de
    ld      (Money),hl
    ld      a,      1
    ld      (CurVal),a
    jp      Display
PopBCInLp:  ; For a later part of the routine.
    pop     bc
    jr      InputLoop
CursorIn:
    ld      hl,     PosnData-1
    ld      bc,     Posn1-1
InNextRow:
    inc     hl
    ld      d,      (hl)
InputLoop:
    inc     hl
    ld      a,      (hl)
    or      a
    jp      z,      KeyInput    ; Null? We went through all the data! No good!
    jp      m,      InNextRow   ; Negative (Bit 7)? End of row.
    ld      e,      (hl)        ; (E, D) equals the top-left corner of current card.
    ; First, check if card is even available to click on
    inc     hl                  ; Point to the card offset
    inc     bc
    ld      a,      (bc)
    or      a                   ; Affect the flags to see A's value
    jr      z,      InputLoop   ; Zero means there's no card there
    push    hl
    ld      l,      (hl)        ; HL = offset to cards that may cover this one
    ld      h,      0
    add     hl,     bc          ; Add them, HL = pointer to card that covers this one
    ld      a,      (hl)
    inc     hl                  ; HL = pointer to second card that covers this one
    ld      h,      (hl)
    cp      h                   ; The two cards will only be equal if they're both zero
    pop     hl
    jr      nz,     InputLoop   ; If not, this card is covered up.
    ; Now to check if cursor is in range
    push    bc
    ld      bc,     (CurPosn)   ; (L, H) holds the position of the cursor.
    ld      a,      d           ; Start with y.
    cp      b
    jr      nc,     PopBCInLp   ; If H < D, position is too far up
    add     a,      12
    cp      b
    jr      c,      PopBCInLp
    ld      a,      e
    cp      c
    jr      nc,     PopBCInLp
    add     a,      7
    cp      c
    jr      c,      PopBCInLp
    ; OK, cursor in range. Check value of card to see if it's valid.
    pop     de
    ld      a,      (de)        ; The value of this card
    and     Face
    ld      l,      a           ; Put it in L
    ld      a,      (Pile)      ; Fetch the value on top of the pile
    and     Face                ; Only need the face value.
    ld      h,      a           ; Back it up
    dec     a                   ; Check if L is one less than A
    jr      nz,     LessThan    ; If Z, A = 0 and should equal $D
    ld      a,      $D          ; The value for King
LessThan:
    cp      l
    jr      z,      Match       ; Good value, put it on pile
    ld      a,      h           ; Restore the card
    inc     a                   ; Check if L is one more than A
    cp      $E                  ; If A = $E, A should equal $1
    jr      nz,     MoreThan
    ld      a,      $1
MoreThan:
    cp      l
    jp      nz,     KeyInput    ; No good? Forget it ever happened
Match:
    ld      a,      (de)
    ld      (Pile), a
    xor     a
    ld      (de),   a
    ld      hl,     (Money)
    push    hl
    ld      hl,     Posn1-1       ; Check if it's a peak (if so, bonus $15)
    ld      b,      3
PeakLoop:
    inc     hl
    call    _cphlde
    jr      z,      Peak
    djnz    PeakLoop
    pop     hl
AddMoney:
    ld      a,      (CurVal)
    ld      c,      a
 ;   ld      b,      0
    add     hl,     bc
    ld      (Money),hl
    inc     a
    ld      (CurVal),a
    jp      Display
Peak:
    pop     hl
    ld      bc,     $000F       ; 15
    add     hl,     bc
    call    ClearedTriPeaks     ; Check if it's the last peak
    jr      nz,     AddMoney    ; If so, another $15!
    add     hl,     bc
    jr      AddMoney

Exit:
    ld      hl,     SaveName
    rst     20h
    rst     10h
    jr      nc,     copydata    ; If tpsave exists, copy data now.
    ld      hl,     tempData - SaveGame ; This is the length of the save game
    call    _createstrng
copydata:
    ld      a,      b           ; Again, move BDE to AHL
    ex      de,     hl
    call    _ahl_plus_2_pg3     ; AHL + 2
    call    _set_abs_dest_addr  ; This time it's the destination
    xor     a
    ld      hl,     SaveGame    ; SaveGame is our source
    call    _set_abs_src_addr
    ld      hl,     tempData - SaveGame ; Length
    call    _set_mm_num_bytes
    call    _mm_ldir            ; Do the copy.
Quit:
    res     4,      (IY+onflags)    ; Eliminates the ERROR:BREAK if user hits ON
    res     donePrgm,(iy+doneflags) ; Forces TI-OS not to display "Done" after exit
    call    _clrScrn                ; Clear LCD and text shadow.
    jp      _homeup                 ; Put cursor in top-left corner and return.


; <<< Routines >>>


MoveDown:
    ld      de,     (CurPosn)
    ld      a,      d
    cp      54
    ret     nc
    call    DrawCursor
    inc     d
    ld      (CurPosn),de
    jr      DrawCursor
MoveLeft:
    ld      de,     (CurPosn)
    ld      a,      e
    or      a
    ret     z
    call    DrawCursor
    dec     e
    ld      (CurPosn),de
    jr      DrawCursor
MoveRight:
    ld      de,     (CurPosn)
    ld      a,      e
    cp      120
    ret     nc
    call    DrawCursor
    inc     e
    ld      (CurPosn),de
    jr      DrawCursor
MoveUp:
    ld      de,     (CurPosn)
    ld      a,      d
    or      a
    ret     z
    call    DrawCursor
    dec     d
    ld      (CurPosn),de
;   jr      DrawCursor

DrawCursor:
    ld      hl,     Cursor
    ld      b,      10
    push    bc
    push    de
    call    PutSprite
    pop     de
    pop     bc
    ret
    

; This is a routine only because it is used so much.
; Checks if the user cleared everything (game over).
; Input:    nothing
; Output:   Z if everything gone
;           NZ if something is still there
ClearedTriPeaks:
    ld      a,      (Posn1)
    or      a
    ret     nz
    ld      a,      (Posn2)
    or      a
    ret     nz
    ld      a,      (Posn3)
    or      a
    ret         ; It doesn't matter whether it's Z or NZ, it'll just return anyway


; DispHL - Displays HL as a Positive Or Negative decimal number
; Uses Two's Complement to get negative numbers.
; Disp_A - Displays A as a decimal number. Always positive for size.
Disp_A:
    ld      h,      0           ; Just use DispHL for speed.
    ld      l,      a
DispHL:
    ld      a,      h
;   Normally, in this routine I compare with $80 to get a range
;   from -32768 to 32767. However, to work better with the display
;   (and to add to how long a game can last), I compare with $D9 to
;   change the range from -9984 to 55551. When people lose money, they
;   will probably become discouraged after about -1056, and won't bother
;   to go deeper. And, for people intent on playing the game, they
;   can continue to play after they hit $32767 for quite a while.
;   cp      $80                 ; Check if HL is negative
    cp      $D9     ; Use mostly positive range ($D900 is the cutoff instead of $8000)
    push    af                  ; Save our result
    jr      c,      Positive    ; Skip to displaying the number if positive
    ex      de,     hl
    ld      hl,     $FFFF
    sbc     hl,     de          ; Subtract HL from $FFFF
    inc     hl                  ; And add 1
Positive:
    xor     a                   ; Clear A
    ld      de,     -1          ; Give it phony co-ords so it doesn't really
    ld      (_curRow),de        ; display anything.
    call    _dispAHL            ; Make TI-OS generate the number for us
    dec     hl                  ; HL points to OP1 where the string is stored.
    pop     af                  ; F holds whether HL is negative
    jr      c,      StillPositive   ; Positive? Display as normal
    ld      (hl),   $2D         ; Load into the first byte a minus sign (-)
StillPositive:
    jp      _vputs              ; Jump to _vputs to display.


; This is the same as DrawRect, but it goes around the outside
; of the given coordinates (easy for the cards).
; For space, the correct length/width *must* be given.
DrawRectOutside:
    dec     d
    dec     e

; DrawRect ~ Draws a rectangle on screen and clears the inside of it
; Input:  E = x   C = width
;         D = y   B = height
; Output: A rectangle on screen
;         Preserves E
;         D is updated to current line
DrawRect:
    push    bc
    call    Border
    pop     bc
    dec     b
    dec     b
RectLoop:
    push    bc
    inc     d
    call    Inside
    pop     bc
    djnz    RectLoop
    inc     d
Border:
    ld      b,      c
    call    LeftEdge
    or      c
    ld      (hl),   a
    push    hl
    call    RightEdge
    or      c
    ld      (hl),   a
    xor     a
    dec     a
Fill:
    pop     bc
    sbc     hl,     bc
    ld      h,      b
    ld      b,      l
    ld      l,      c
    dec     b
    ret     z
FillLoop:
    inc     hl
    ld      (hl),   a
    djnz    FillLoop
    ret

Inside:
    ld      b,      c
    call    LeftEdge
    inc     a
    srl     a
    or      c
    ld      (hl),   a
    push    hl
    call    RightEdge
    cpl
    inc     a
    or      c
    ld      (hl),   a
    xor     a
    jr      Fill

LeftEdge:
    xor     a
    push    bc
    push    af
    call    FindPixelCommon
    ld      b,      a
    pop     af
LeftEndLoop:
    scf
    rra
    djnz    LeftEndLoop
    pop     bc
FinishEdge:
    push    af
    and     (hl)
    ld      c,      a
    pop     af
    cpl
    ret

RightEdge:
    push    de
    ld      a,      e
    add     a,      b
    ld      e,      a
    xor     a
    dec     a
    push    bc
    push    af
    call    FindPixelCommon
    ld      b,      a
    pop     af
RightEndLoop:
    srl     a
    djnz    RightEndLoop
    pop     bc
    pop     de
    jr      FinishEdge

; PutSprite ~ Puts a 8 by x sprite
; E = x         HL = ptr to sprite
; D = y         B = # Rows
PutSprite:
    push    hl              ; Save the sprite pointer
    pop     ix              ; Move sprite pointer to IX
    push    bc              ; Save # rows
    call    FindPixelCommon ; This will calculate the byte & how far to rotate
    pop     bc              ; Get our row counter back
    jr      z,      Aligned ; In that case, handle it faster....see Aligned.
    ld      c,      a       ; A backup of the bit.
    ld      de,     15      ; A row is 16 bytes, minus one cuz it's two bytes.
RowLoop:
    push    bc              ; Save the row counter and bit
    ld      b,      c       ; The bit is in B as well
    ld      a,      (ix)    ; A holds a byte from the sprite.
    ld      c,      0       ; Clear C
ShiftLoop:
    srl     a               ; Shift part of A into C
    rr      c               ; The current byte of sprite is split between A and C
    djnz    ShiftLoop       ; Continue until it's shifted correctly between the two.
    xor     (hl)            ; Or A with (HL)
    ld      (hl),   a       ; Load it into memory
    inc     hl              ; Move to next byte of memory
    ld      a,      c       ; Move to next byte of sprite
    xor     (hl)            ; Do the same as before
    ld      (hl),   a       
    add     hl,     de      ; Move to next row of memory
    inc     ix              ; Move to next row of sprite
    pop     bc              ; Get our row counter again
    djnz    RowLoop         ; Keep going until we reach the end
    ret                     ; We are done!
Aligned:
    ld      de,     16      ; This time what we're displaying is only 1 byte
PutSLoop:
    ld      a,      (ix)
    xor     (hl)
    ld      (hl),   a
    inc     ix
    add     hl,     de
    djnz    PutSLoop
    ret

; As my DrawRect routine morphed and changed, this routine was no longer
; necessary to support it, but I include it as an example.

; FindPixel ~ Finds a pixel on the screen
; In: DE = (y, x) 
; Out:  HL = byte in video mem
;       A  = bitmask for that byte
;       DE = (y, x)
;FindPixel:
;    call    FindPixelCommon
;    ld      b,      a       ; This may be used as our counter
;    ld      a,      %10000000   ; This is the current bitmask
;    ret     z               ; If the AND 7 was zero, the bitmask is $80. Return.
;RotateLoop:
;    rrca
;    djnz    RotateLoop
;    ret

; FindPixelCommon
; In:  DE = (y, x)
; Out: HL = byte in video memory
;      A  = # of times to rotate
;      Z if pixel is even with byte
;      DE = (y, x)
FindPixelCommon:
    ld      h,      0       ; The first thing we need to do is multiply
    ld      l,      d       ; y by 16. Now HL = y
    add     hl,     hl      ; y x 2
    add     hl,     hl      ; y x 4
    add     hl,     hl      ; y x 8
    add     hl,     hl      ; y x 16
    ld      c,      e       ; Now to divide x by 8.
    srl     c               ; x / 2
    srl     c               ; x / 4
    srl     c               ; x / 8
    ld      b,      0       ; Now to add these two
    add     hl,     bc      ; Now we have our offset
    ld      bc,     $FC00   ; Video memory starts at $FC00
    add     hl,     bc      ; Add these and we have our starting byte.
    ld      a,      e       ; Now we should modulo x with 8 to find the bit.
    and     %00000111       ; If zero, x is a multiple of 8.
    ret


; <<< Borrowed Routines >>>


; Taken right from ION, since I couldn't make a good routine myself
; Kudos to Joe Wingburmuehle

;-----> Generate a random number
; input b=upper bound
; ouput a=answer 0<=a<b
; all registers are preserved except: af and bc
Random:
        push    hl
        push    de
        ld      hl,(randData)
        ld      a,r
        ld      d,a
        ld      e,(hl)
        add     hl,de
        add     a,l
        xor     h
        ld      (randData),hl
        sbc     hl,hl
        ld      e,a
        ld      d,h
randomLoop:
        add     hl,de
        djnz    randomLoop
        ld      a,h
        pop     de
        pop     hl
        ret


; <<< Permanent Data >>>

Title:
    .db $DB,$DB,"TriPeaks",$DB,0

GameOver:
    .db "Press F1 to deal",0

SaveName equ $-1
    .db 6,"tpsave"

Faces:
    .db "AJQK"

PosnData:
; This is the data the program uses to place the cards on the screen.
; The first byte in each row is the y coordinate.
; The x coordinates and offsets for each card continue the row.
; The offset is how far away the pair of cards are that cover the current one.
; The final byte is a key saying end of row. (Bit 7 set)
; A null signifies end of data. That null is the first byte of the next sprite.
    .db $03,$18,$03,$3C,$04,$60,$05,$80
    .db $0A,$12,$06,$1E,$06,$36,$07,$42,$07,$5A,$08,$66,$08,$80
    .db $11,$0C,$09,$18,$09,$24,$09,$30,$09,$3C,$09,$48,$09,$54,$09,$60,$09,$6C,$09,$80
    .db $18,$06,$26,$12,$25,$1E,$24,$2A,$23,$36,$22,$42,$21,$4E,$20,$5A,$1F,$66,$1E,$72,$1D,$80
    .db $29,$18,$1C ; ,0

; Sprites

CardBack:
.db %00000000
.db %00000000
.db %00010000
.db %00101010
.db %01000101
.db %10101000
.db %00011000
.db %00010000
.db %00001000
.db %00000100
.db %00000100
.db %00000010
.db %00000001

SuitSprites:

Club:
.db %00011100
.db %00011100
.db %00111110
.db %00111110
.db %00001000
.db %00011100

Diamond:
.db %00001000
.db %00010100
.db %00100010
.db %00010100
.db %00001000
.db %00000000

Spade:
.db %00001000
.db %00011100
.db %00111110
.db %00111110
.db %00001000
.db %00011100

Heart:
.db %00010100
.db %00101010
.db %00100010
.db %00010100
.db %00001000
.db %00000000

Cursor:
.db %10000000
.db %11000000
.db %11100000
.db %11110000
.db %11111000
.db %11111100
.db %11111110
.db %11111100
.db %11100000
.db %11000000

; <<< Save Game Area >>>
SaveGame    equ $

Money   equ $       ; 2 bytes (Max $55551, min $-9984)
CurVal  equ $+2     ; Current $$ of a card from the TriPeaks
DeckLen equ $+3     ; Length of deck
Posn1   equ $+4     ; Positions on screen
Posn2   equ $+5
Posn3   equ $+6
Posn4   equ $+7
Posn5   equ $+8
Posn6   equ $+9
Posn7   equ $+10
Posn8   equ $+11
Posn9   equ $+12
Posn10  equ $+13
Posn11  equ $+14
Posn12  equ $+15
Posn13  equ $+16
Posn14  equ $+17
Posn15  equ $+18
Posn16  equ $+19
Posn17  equ $+20
Posn18  equ $+21
Posn19  equ $+22
Posn20  equ $+23
Posn21  equ $+24
Posn22  equ $+25
Posn23  equ $+26
Posn24  equ $+27
Posn25  equ $+28
Posn26  equ $+29
Posn27  equ $+30
Posn28  equ $+31
Pile    equ $+32
Deck    equ $+33

; <<< Temporary Data >>>

tempData equ $+56

CurPosn  equ tempData
CursorY equ tempData
CursorX equ tempData+1
KeyGrp  equ tempData+2
KeyPres equ tempData+3
DumpR   equ tempData+4    ; This is used for the null for default-flipped up cards
ScrapL  equ tempData+5    ; DumpR and ScrapL are both 0 when game is in session.
Scrap   equ tempData+6    ; Scrap deck
randData    equ tempData+58 ;randData

.end
